//Warren Fong
//CSC 290
//Secure Chat

import java.net.*;
import java.io.*;
import java.applet.*;
import java.awt.*;
import java.math.BigInteger;

public class ChatClient extends Applet
{  private Socket socket              = null;
   private DataInputStream  console   = null;
   private DataOutputStream streamOut = null;
   private ChatClientThread client    = null;
   private TextArea  display = new TextArea();
   private TextField input   = new TextField();
   private Button    send    = new Button("Send"), connect = new Button("Connect"),
   quit    = new Button("Bye");
   private String    serverName = "localhost";
   private int       serverPort = 1024;
   private BigInteger pubKey, privKey, Evalue;
   
   public void init() {
       Panel keys = new Panel();
       keys.setLayout(new GridLayout(1,2));
       keys.add(quit);
       keys.add(connect);
       Panel south = new Panel();
       south.setLayout(new BorderLayout());
       south.add("West", keys);
       south.add("Center", input);
       south.add("East", send);
       Label title = new Label("Secure RSA Chat Client Applet", Label.CENTER);
       title.setFont(new Font("Helvetica", Font.BOLD, 14));
       setLayout(new BorderLayout(10, 10));
       add("North", title); add("Center", display);
       add("South",  south);
       input.setText("Enter Login here and click Connect");
       quit.disable();
       send.disable();
       getParameters(); }
   
   public boolean action(Event e, Object o)
   {  if (e.target == quit)
      {  input.setText(".bye");
         send();  quit.disable(); send.disable(); connect.enable(); }
      else if (e.target == connect)
      {  connect(serverName, serverPort); }
      else if (e.target == send)
      {  send(); input.requestFocus(); }
      return true; }
   
   public void connect(String serverName, int serverPort)
   {  println("Establishing connection. Please wait ...");
      try
      {  socket = new Socket(serverName, serverPort);
         println("Connected: " + socket);
         open();
         send.enable();
         send();
         println("NameEntered");
         println("KeysGenerated");
         keyGen();
         connect.disable();
         quit.enable(); }
      catch(UnknownHostException uhe)
      {  println("Host unknown: " + uhe.getMessage()); }
      catch(IOException ioe)
      {  println("Unexpected exception: " + ioe.getMessage()); } }
   
   private void send()
   {  try
      {  streamOut.writeUTF(input.getText()); streamOut.flush(); input.setText(""); }
      catch(IOException ioe)
      {  println("Sending error: " + ioe.getMessage()); close(); } }
   
   private void sendKey()
   {  try {
          streamOut.writeUTF(pubKey.toString());streamOut.flush(); input.setText(""); }
      catch(IOException ioe)
      {  println("Sending error: " + ioe.getMessage()); close(); } }
   
   private void sendE() {
       try {
           streamOut.writeUTF(Evalue.toString());
           streamOut.flush();
           input.setText(""); }
       catch(IOException ioe)
       {  println("Sending error: " + ioe.getMessage()); close(); }
       
   }
   
   public void handle(String msg)
   {  if (msg.equals(".bye"))
      {  println("Good bye. Press RETURN to exit ...");  close(); }
      else {
          println(msg);
          //println("before in");
          printDecrypted(msg);
          
      }
   }
   
   public void keyGen() {
       //println("get in there");
       int bitVal = 192;
       Rsa generated = new Rsa(bitVal);
       println("N:");
       printBigInt(generated.Nvalue);
       pubKey = generated.Nvalue;
       println("E:");
       printBigInt(generated.Evalue);
       Evalue = generated.Evalue;
       //printBigInt(Evalue);
       println("D:");
       printBigInt(generated.Dvalue);
       privKey = generated.Dvalue;
       sendKey();
       println("keySent");
       sendE();
       println("eSent");
   }
   public void open()
   {  try
      {  streamOut = new DataOutputStream(socket.getOutputStream());
         client = new ChatClientThread(this, socket); }
      catch(IOException ioe)
      {  println("Error opening output stream: " + ioe); } }
   
   public void close()
   {  try
      {  if (streamOut != null)  streamOut.close();
         if (socket    != null)  socket.close(); }
      catch(IOException ioe)
      {  println("Error closing ..."); }
      client.close();  client.stop(); }
   
   private void println(String msg)
   {  display.appendText(msg + "\n"); }
   
   private void printBigInt(BigInteger val)
   { display.appendText(val + "\n");}
   
   private void printDecrypted(String msg) {
       //println("decrypting....");
       //println(msg);
       String dmsg = msg.substring(msg.indexOf(":")+1);
       String tdmsg = dmsg.trim();
       //println(dmsg);
       //println(tdmsg);
       //println(".");
       //Rsa current = new Rsa();
       //println(".");
       // println("beforebig");
       BigInteger ciphertext = new BigInteger(tdmsg);
       //println("Big INT");
       //printBigInt(ciphertext);
       Rsa current = new Rsa();
       BigInteger out = current.decrypt(ciphertext, privKey, pubKey);
       //printBigInt(out);
       String output = new String(out.toByteArray());
       println("Decrypted: "+ output);
   }
   
   public void getParameters()
   {  serverName = getParameter("host");
      serverPort = Integer.parseInt("1024"); }
}

