//Warren Fong
//CSC 290
//Secure Chat

import java.net.*;
import java.io.*;
import java.math.BigInteger;

public class ChatServer implements Runnable
{  private ChatServerThread clients[] = new ChatServerThread[50];
   private ServerSocket server = null;
   private Thread       thread = null;
   private BigInteger user1 = null;
   private BigInteger eUser1 = null;
   private BigInteger user2 = null;
   private BigInteger eUser2 = null;
   String UserName;
   private int clientCount = 0;
   
   public ChatServer(int port)
   {  try
      {  System.out.println("Binding to port " + port + ", please wait  ...");
         server = new ServerSocket(port);
         System.out.println("Server started: " + server);
         start(); }
      catch(IOException ioe)
      {  System.out.println("Can not bind to port " + port + ": " + ioe.getMessage()); }
   }
   public void run()
   {  while (thread != null)
      {  try
         {  System.out.println("Waiting for a client ...");
            addThread(server.accept()); }
         catch(IOException ioe)
         {  System.out.println("Server accept error: " + ioe); stop(); }
      }
   }
   public void start()  { 
       if (thread == null)
      {  thread = new Thread(this); 
         thread.start();
      }
 }
   public void stop()   { 
       if (thread != null)
      {  thread.stop(); 
         thread = null;
      }
 }
   private int findClient(int ID)
   {  for (int i = 0; i < clientCount; i++)
          if (clients[i].getID() == ID)
              return i;
      return -1;
   }
   public synchronized void handle(int ID, String input)
   {  
       Rsa generated = new Rsa();
      if (input.equals(".bye"))
      {  clients[findClient(ID)].send(".bye");
         remove(ID); }
      else
          for (int i = 0; i < clientCount; i++)
          {
              int prev = ID -1;
              int after = ID + 1;
              if(findClient(prev) == -1 ){
                  if(findClient(after) == -1){
                    clients[i].send(clients[findClient(ID)].getUserName() + " 2: " + input);
                  }
                  else{  
                      BigInteger text = new BigInteger(input.getBytes());
                      BigInteger encrypted = generated.encrypt(text, eUser2, user2);
                      String Sencrypted = encrypted.toString();
                      clients[i].send(clients[findClient(ID)].getUserName() + " 1: " + Sencrypted);
                  }
              }
              else{
                  BigInteger text = new BigInteger(input.getBytes());
                  BigInteger encrypted = generated.encrypt(text, eUser1, user1);
                  String Sencrypted = encrypted.toString();
                  clients[i].send(clients[findClient(ID)].getUserName() + " 3: " + Sencrypted);
                }
              }
   }
   public synchronized void initialStor(String Name)
   {
       UserName = Name;
       System.out.println(Name + " connected");

   }
   public synchronized void pubKeyStor(String in)
   {
       BigInteger user = new BigInteger(in);
       if(user2 == null){
           if(user1 == null){
               user1 = user;
               System.out.println(user1);
           }
           else{
               user2 = user;
               System.out.println(user2);
           }
       }   
   }
   public synchronized void pubEStor(String in)
   {
       BigInteger euser = new BigInteger(in);
       if(eUser2 == null){
           if(eUser1 == null){
               eUser1 = euser;
               System.out.println(eUser1);
           }
           else{
               eUser2 = euser;
               System.out.println(eUser2);
           }
       }   
   }
   public synchronized void remove(int ID)
   {  int pos = findClient(ID);
      if (pos >= 0)
      {  ChatServerThread toTerminate = clients[pos];
         System.out.println("Removing client thread " + ID + " at " + pos);
         if (pos < clientCount-1)
             for (int i = pos+1; i < clientCount; i++)
                 clients[i-1] = clients[i];
         clientCount--;
         try
         {  toTerminate.close(); }
         catch(IOException ioe)
         {  System.out.println("Error closing thread: " + ioe); }
         toTerminate.stop(); }
   }
   private void addThread(Socket socket)
   {  if (clientCount < clients.length)
      {  System.out.println("Client accepted: " + socket);
         clients[clientCount] = new ChatServerThread(this, socket);
         try
         {  clients[clientCount].open();
            clients[clientCount].start();
            clientCount++; }
         catch(IOException ioe)
         {  System.out.println("Error opening thread: " + ioe); } }
      else
          System.out.println("Client refused: maximum " + clients.length + " reached.");
   }
   public static void main(String args[]) { 
       ChatServer server = null;
      if (args.length != 1)
         System.out.println("Usage: java ChatServer port");
      else
         server = new ChatServer(Integer.parseInt(args[0]));
    }
}